#!/bin/sh
#
# handy functions:
#
# a function to display a failure message and then exit 
fail ( ) {
	echo -e $@
	exit 1
}

# "get with default" function
# this function prompts the user with a query and default reply
# it returns the user reply
getwd ( ) {
	query="$1"
	default="$2"
	echo -en "$query [$default]" | cat >&2
	read response
	if [ x$response = "x" ]; then
		response=$default
	fi
	echo $response
}

# "get yes no" function
# this function prompts the user with a query and will continue to do so
# until they reply with either "y" or "n"
getyn ( ) {
	query="$@"
	echo -en $query | cat >&2
	read response
	while [ x$response != "xy" -a x$response != "xn" ]; do
		echo -e "\n'y' or 'n' only please...\n" | cat >&2
		echo -en $query | cat >&2
		read response
	done	
	echo $response
}

# Configuration 
# Set defaults
defaultusername="scponly"
defaulthomedirprefix="/home"
defaultwriteabledir="incoming"

osname=`uname -s | tr ' ' '_'`
# Pathname to platform/OS specific setup scripts
prescript="build_extras/arch/$osname.pre.sh"
postscript="build_extras/arch/$osname.post.sh"

# The following is a list of binaries that will be staged in the target dir
BINARIES=`/usr/bin/grep '#define PROG_' config.h | /usr/bin/cut -f2 -d\" | /usr/bin/grep -v ^cd$`

# We set the install path in a variable so the presetup script can overwrite it on systems
# which require it
INSTALL_PATHNAME="/usr/bin/install -c -o root -g bin"

# Attempt a best guess at required libs, we can append things in the presetup script if we need to
LDSOFOUND=0

# Default to useradd, not pw
USE_PW=0

# We must have ldd to work
if [ x/usr/bin/ldd = x ]; then
	echo "This script requires the program ldd to determine which"
	fail "shared libraries to copy into your chrooted dir..."
fi
LIB_LIST=`/usr/bin/ldd $BINARIES 2> /dev/null | /usr/bin/cut -f7 -d" " | /usr/bin/grep -v "$1:" | /usr/bin/grep -v "^$" | /usr/bin/sort -u`

# Also require ld.so
LDSO_LIST="/lib/ld.so /libexec/ld-elf.so /libexec/ld-elf.so.1 /usr/libexec/ld.so /lib/ld-linux.so.2 /usr/libexec/ld-elf.so.1"
for lib in $LDSO_LIST; do
	if [ -f $lib ]; then
		LDSOFOUND=1;
		LIB_LIST="$LIB_LIST $lib"
	fi
done

# I've since forgotten which OS this is for, it should be relocated to a presetup script
/bin/ls /lib/libnss_compat* > /dev/null 2>&1 
if [ $? -eq 0 ]; then
	LIB_LIST="$LIB_LIST /lib/libnss_compat*"
fi

# Check that the configure options are correct for chrooted operation:
if [ xscponlyc = x ] || [ ! -f ./config.h ]; then
	echo 
	echo 'Your scponly build is not configured for chrooted operation.'
	echo 'please reconfigure as follows, then rebuild and reinstall:'
	echo
	echo './configure --enable-chrooted-binary (... other options)'
	echo
	exit 1
fi

# Ensure we have useradd
if [ x/usr/sbin/useradd = x ]; then
    if [ x = x ]; then
		echo "This script requires the program useradd or pw to add your"
		fail "chrooted scponly user."
	else
  		USE_PW=1;
    fi
fi

# we need to be root
if [ `/usr/bin/id -u` != "0" ]; then
	fail "You must be root to run this script\n"
fi

echo
echo Next we need to set the home directory for this scponly user.
echo please note that the user\'s home directory MUST NOT be writeable
echo by the scponly user.  this is important so that the scponly user
echo cannot subvert the .ssh configuration parameters.
echo
echo for this reason, a writeable subdirectory will be created that
echo the scponly user can write into.  
echo

# We're installing a new user by default
method="create"

targetuser=`getwd "Username to install" "$defaultusername"`
username_collision=`id $targetuser > /dev/null 2> /dev/null; echo $?`
if [ $username_collision -eq 0 ] ; then

	echo ""
	echo "Warning:"
	echo "The user $targetuser already exists.  You are either adding a"
	echo "user that already exists, or are upgrading an older user to "
	echo "scponly."
	echo ""
	continue=`getyn "Do you with to continue? [y/n] "`
	if [ "$continue" = "n" ]; then
		fail "Try a username that doesn't exist, or delete this user"
	fi

	# If we've made it here, we're upgrading an existing user
	method="upgrade"
fi 

targetdir=`getwd "Home directory for this user" "$defaulthomedirprefix/$targetuser"`
writeabledir=`getwd "Name of writeable subdirectory" "$defaultwriteabledir"`

# Do we run an OS specific pr-setup script?
if [ -f "$prescript" ]; then
	. "$prescript"
fi

# If neither the presetup script or the best guess could find 
# ld.so, we have to bail here
if [ $LDSOFOUND -eq 0 ]; then
	fail "Unable cant find your equivalent of ld.so"
fi

# ACTUAL MODIFICATIONS BEGIN HERE

# Now start doing things.
echo ""
echo "Creating directories:"

# This part shouldnt strictly be required, but i'll leave it 
# in until i'm sure of it
if [ ! -d $targetdir ]; then
	echo "\t- $targetdir"
	$INSTALL_PATHNAME -d $targetdir
	/bin/chmod 755 $targetdir
fi

if [ ! -d $targetdir/etc ]; then
	echo "\t- $targetdir/etc"
	$INSTALL_PATHNAME -d $targetdir/etc
	/usr/sbin/chown 0:0 $targetdir/etc
	/bin/chmod 755 $targetdir/etc
fi

# Ensure that we have the .ssh folder
if [ ! -d $targetdir/.ssh ]; then
	echo "\t- $targetdir/.ssh"
	$INSTALL_PATHNAME -d $targetdir/.ssh
	/usr/sbin/chown 0:0 $targetdir/.ssh
fi

# Ensure that we have all the binaries that we need in place
echo "Installing binaries:"
for bin in $BINARIES; do
	echo "\t- $targetdir$bin"
	$INSTALL_PATHNAME -d $targetdir/`/usr/bin/dirname $bin`
	$INSTALL_PATHNAME $bin $targetdir$bin
done

# Ensure that we have all the libraries in place
echo "Installing libraries:"
if [ "x$LIB_LIST" != "x" ]; then
	for lib in $LIB_LIST; do
		echo "\t- $targetdir$lib"
		$INSTALL_PATHNAME -d $targetdir/`/usr/bin/dirname $lib`
		$INSTALL_PATHNAME $lib $targetdir/$lib
	done
fi

echo ""

# We only perform these steps if we're creating a new user account
if [ "$method" = "create" ]; then
	if [ "x$USE_PW" = x0 ] ; then
	    /usr/sbin/useradd -d "$targetdir" -s "${DESTDIR}/usr/local/sbin/scponlyc" $targetuser
	    if [ $? -ne 0 ]; then
	         fail "If this user exists, remove it and try again"
	    fi
	else
	     useradd -n $targetuser -s "${DESTDIR}/usr/local/sbin/scponlyc" -d "$targetdir"
	    if [ $? -ne 0 ]; then
	         fail "If this user exists, remove it and try again"
	    fi
	fi 
fi

# The target directory must be root owned
/usr/sbin/chown 0:0 $targetdir 

# The writeable directory must be owned by the user:group
if [ ! -d $targetdir/$writeabledir ]; then
	echo -e "\nCreating $targetdir/$writeabledir directory for uploading files"
	$INSTALL_PATHNAME -o $targetuser -d $targetdir/$writeabledir
fi

#
#	set the perms on the writeable dir so that the new user owns it
#
newuid=`/usr/bin/id -u $targetuser`
newgid=`/usr/bin/id -g $targetuser`
/usr/sbin/chown $newuid:$newgid $targetdir/$writeabledir

# Do we have an OS specific post-install script?
if [ -f "$postscript" ]; then
    . "$postscript"
else
	# Otherwise, revert to the old "best guess" system, which sucks
	echo ""
	echo "Your platform ($osname) does not have a platform specific setup script."
	echo "This install script will attempt a best guess."
	echo "If you perform customizations, please consider sending me your changes."
	echo "Look to the templates in build_extras/arch."
	echo " - joe at sublimation dot org"
	echo
	if [ x/usr/sbin/pwd_mkdb = x ]; then
		# Ok we dont have pwd_mkdb, lets improvise:
		/usr/bin/grep $targetuser /etc/passwd > $targetdir/etc/passwd
	else
		# This is for systems which do have pwd_mkdb
		/usr/bin/grep $targetuser /etc/master.passwd > $targetdir/etc/master.passwd
		/usr/sbin/pwd_mkdb -d "$targetdir/etc" $targetdir/etc/master.passwd
		/bin/rm -rf $targetdir/etc/master.passwd $targetdir/etc/spwd.db
	fi
fi

# Finally, set the password, but only for new accounts
if [ "$method" = "create" ]; then
	passwd $targetuser
fi

echo ""
echo "If you experience a warning with winscp regarding groups, please install"
echo "the provided hacked out fake groups program into your chroot, like so:"
echo ""
echo " # cp /usr/local/share/scponly/groups $targetdir/bin/groups" 
echo ""
echo "scponly user setup complete"
